/* Automatically generated. Do not edit. */
#ifndef __ASM_OFFSETS_H__
#define __ASM_OFFSETS_H__

#include <linux/config.h>

#if defined(__KERNEL__) && !defined(__ASSEMBLY__)
# if !((__GNUC__ > 2) || (__GNUC__ == 2 && __GNUC_MINOR__ >= 8))
#  error Please issue 'make check_asm' in linux top-level directory first
# endif
#endif

#ifndef CONFIG_SMP

#define AOFF_task_state	0x00000000
#define ASIZ_task_state	0x00000008
#define AOFF_task_flags	0x00000008
#define ASIZ_task_flags	0x00000008
#define AOFF_task_sigpending	0x00000010
#define ASIZ_task_sigpending	0x00000004
#define AOFF_task_addr_limit	0x00000014
#define ASIZ_task_addr_limit	0x00000001
#define AOFF_task_exec_domain	0x00000018
#define ASIZ_task_exec_domain	0x00000008
#define AOFF_task_need_resched	0x00000020
#define ASIZ_task_need_resched	0x00000008
#define AOFF_task_counter	0x00000028
#define ASIZ_task_counter	0x00000008
#define AOFF_task_priority	0x00000030
#define ASIZ_task_priority	0x00000008
#define AOFF_task_avg_slice	0x00000038
#define ASIZ_task_avg_slice	0x00000008
#define AOFF_task_has_cpu	0x00000040
#define ASIZ_task_has_cpu	0x00000004
#define AOFF_task_processor	0x00000044
#define ASIZ_task_processor	0x00000004
#define AOFF_task_last_processor	0x00000048
#define ASIZ_task_last_processor	0x00000004
#define AOFF_task_lock_depth	0x0000004c
#define ASIZ_task_lock_depth	0x00000004
#define AOFF_task_next_task	0x00000050
#define ASIZ_task_next_task	0x00000008
#define AOFF_task_prev_task	0x00000058
#define ASIZ_task_prev_task	0x00000008
#define AOFF_task_run_list	0x00000060
#define ASIZ_task_run_list	0x00000010
#define AOFF_task_binfmt	0x00000070
#define ASIZ_task_binfmt	0x00000008
#define AOFF_task_exit_code	0x00000078
#define ASIZ_task_exit_code	0x00000004
#define AOFF_task_exit_signal	0x0000007c
#define ASIZ_task_exit_signal	0x00000004
#define AOFF_task_pdeath_signal	0x00000080
#define ASIZ_task_pdeath_signal	0x00000004
#define AOFF_task_personality	0x00000088
#define ASIZ_task_personality	0x00000008
#define AOFF_task_pid	0x00000094
#define ASIZ_task_pid	0x00000004
#define AOFF_task_pgrp	0x00000098
#define ASIZ_task_pgrp	0x00000004
#define AOFF_task_tty_old_pgrp	0x0000009c
#define ASIZ_task_tty_old_pgrp	0x00000004
#define AOFF_task_session	0x000000a0
#define ASIZ_task_session	0x00000004
#define AOFF_task_leader	0x000000a4
#define ASIZ_task_leader	0x00000004
#define AOFF_task_p_opptr	0x000000a8
#define ASIZ_task_p_opptr	0x00000008
#define AOFF_task_p_pptr	0x000000b0
#define ASIZ_task_p_pptr	0x00000008
#define AOFF_task_p_cptr	0x000000b8
#define ASIZ_task_p_cptr	0x00000008
#define AOFF_task_p_ysptr	0x000000c0
#define ASIZ_task_p_ysptr	0x00000008
#define AOFF_task_p_osptr	0x000000c8
#define ASIZ_task_p_osptr	0x00000008
#define AOFF_task_pidhash_next	0x000000d0
#define ASIZ_task_pidhash_next	0x00000008
#define AOFF_task_pidhash_pprev	0x000000d8
#define ASIZ_task_pidhash_pprev	0x00000008
#define AOFF_task_wait_chldexit	0x000000e0
#define ASIZ_task_wait_chldexit	0x00000020
#define AOFF_task_vfork_sem	0x00000100
#define ASIZ_task_vfork_sem	0x00000008
#define AOFF_task_policy	0x00000108
#define ASIZ_task_policy	0x00000008
#define AOFF_task_rt_priority	0x00000110
#define ASIZ_task_rt_priority	0x00000008
#define AOFF_task_it_real_value	0x00000118
#define ASIZ_task_it_real_value	0x00000008
#define AOFF_task_it_prof_value	0x00000120
#define ASIZ_task_it_prof_value	0x00000008
#define AOFF_task_it_virt_value	0x00000128
#define ASIZ_task_it_virt_value	0x00000008
#define AOFF_task_it_real_incr	0x00000130
#define ASIZ_task_it_real_incr	0x00000008
#define AOFF_task_it_prof_incr	0x00000138
#define ASIZ_task_it_prof_incr	0x00000008
#define AOFF_task_it_virt_incr	0x00000140
#define ASIZ_task_it_virt_incr	0x00000008
#define AOFF_task_real_timer	0x00000148
#define ASIZ_task_real_timer	0x00000028
#define AOFF_task_times	0x00000170
#define ASIZ_task_times	0x00000020
#define AOFF_task_start_time	0x00000190
#define ASIZ_task_start_time	0x00000008
#define AOFF_task_per_cpu_utime	0x00000198
#define ASIZ_task_per_cpu_utime	0x00000008
#define AOFF_task_min_flt	0x000001a8
#define ASIZ_task_min_flt	0x00000008
#define AOFF_task_maj_flt	0x000001b0
#define ASIZ_task_maj_flt	0x00000008
#define AOFF_task_nswap	0x000001b8
#define ASIZ_task_nswap	0x00000008
#define AOFF_task_cmin_flt	0x000001c0
#define ASIZ_task_cmin_flt	0x00000008
#define AOFF_task_cmaj_flt	0x000001c8
#define ASIZ_task_cmaj_flt	0x00000008
#define AOFF_task_cnswap	0x000001d0
#define ASIZ_task_cnswap	0x00000008
#define AOFF_task_uid	0x000001dc
#define ASIZ_task_uid	0x00000004
#define AOFF_task_euid	0x000001e0
#define ASIZ_task_euid	0x00000004
#define AOFF_task_suid	0x000001e4
#define ASIZ_task_suid	0x00000004
#define AOFF_task_fsuid	0x000001e8
#define ASIZ_task_fsuid	0x00000004
#define AOFF_task_gid	0x000001ec
#define ASIZ_task_gid	0x00000004
#define AOFF_task_egid	0x000001f0
#define ASIZ_task_egid	0x00000004
#define AOFF_task_sgid	0x000001f4
#define ASIZ_task_sgid	0x00000004
#define AOFF_task_fsgid	0x000001f8
#define ASIZ_task_fsgid	0x00000004
#define AOFF_task_ngroups	0x000001fc
#define ASIZ_task_ngroups	0x00000004
#define AOFF_task_groups	0x00000200
#define ASIZ_task_groups	0x00000080
#define AOFF_task_cap_effective	0x00000280
#define ASIZ_task_cap_effective	0x00000004
#define AOFF_task_cap_inheritable	0x00000284
#define ASIZ_task_cap_inheritable	0x00000004
#define AOFF_task_cap_permitted	0x00000288
#define ASIZ_task_cap_permitted	0x00000004
#define AOFF_task_user	0x00000290
#define ASIZ_task_user	0x00000008
#define AOFF_task_rlim	0x00000298
#define ASIZ_task_rlim	0x000000a0
#define AOFF_task_used_math	0x00000338
#define ASIZ_task_used_math	0x00000002
#define AOFF_task_comm	0x0000033a
#define ASIZ_task_comm	0x00000010
#define AOFF_task_link_count	0x0000034c
#define ASIZ_task_link_count	0x00000004
#define AOFF_task_tty	0x00000350
#define ASIZ_task_tty	0x00000008
#define AOFF_task_semundo	0x00000358
#define ASIZ_task_semundo	0x00000008
#define AOFF_task_semsleeping	0x00000360
#define ASIZ_task_semsleeping	0x00000008
#define AOFF_task_thread	0x00000370
#define ASIZ_task_thread	0x00000460
#define AOFF_task_fs	0x000007d0
#define ASIZ_task_fs	0x00000008
#define AOFF_task_files	0x000007d8
#define ASIZ_task_files	0x00000008
#define AOFF_task_mm	0x000007e0
#define ASIZ_task_mm	0x00000008
#define AOFF_task_active_mm	0x000007e8
#define ASIZ_task_active_mm	0x00000008
#define AOFF_task_sigmask_lock	0x000007f0
#define ASIZ_task_sigmask_lock	0x00000000
#define AOFF_task_sig	0x000007f0
#define ASIZ_task_sig	0x00000008
#define AOFF_task_signal	0x000007f8
#define ASIZ_task_signal	0x00000008
#define AOFF_task_blocked	0x00000800
#define ASIZ_task_blocked	0x00000008
#define AOFF_task_sigqueue	0x00000808
#define ASIZ_task_sigqueue	0x00000008
#define AOFF_task_sigqueue_tail	0x00000810
#define ASIZ_task_sigqueue_tail	0x00000008
#define AOFF_task_sas_ss_sp	0x00000818
#define ASIZ_task_sas_ss_sp	0x00000008
#define AOFF_task_sas_ss_size	0x00000820
#define ASIZ_task_sas_ss_size	0x00000008
#define ASIZ_task	0x00000830
#define AOFF_mm_mmap	0x00000000
#define ASIZ_mm_mmap	0x00000008
#define AOFF_mm_mmap_avl	0x00000008
#define ASIZ_mm_mmap_avl	0x00000008
#define AOFF_mm_mmap_cache	0x00000010
#define ASIZ_mm_mmap_cache	0x00000008
#define AOFF_mm_pgd	0x00000018
#define ASIZ_mm_pgd	0x00000008
#define AOFF_mm_mm_users	0x00000020
#define ASIZ_mm_mm_users	0x00000004
#define AOFF_mm_mm_count	0x00000024
#define ASIZ_mm_mm_count	0x00000004
#define AOFF_mm_map_count	0x00000028
#define ASIZ_mm_map_count	0x00000004
#define AOFF_mm_mmap_sem	0x00000030
#define ASIZ_mm_mmap_sem	0x00000030
#define AOFF_mm_page_table_lock	0x00000060
#define ASIZ_mm_page_table_lock	0x00000000
#define AOFF_mm_context	0x00000060
#define ASIZ_mm_context	0x00000008
#define AOFF_mm_start_code	0x00000068
#define ASIZ_mm_start_code	0x00000008
#define AOFF_mm_end_code	0x00000070
#define ASIZ_mm_end_code	0x00000008
#define AOFF_mm_start_data	0x00000078
#define ASIZ_mm_start_data	0x00000008
#define AOFF_mm_end_data	0x00000080
#define ASIZ_mm_end_data	0x00000008
#define AOFF_mm_start_brk	0x00000088
#define ASIZ_mm_start_brk	0x00000008
#define AOFF_mm_brk	0x00000090
#define ASIZ_mm_brk	0x00000008
#define AOFF_mm_start_stack	0x00000098
#define ASIZ_mm_start_stack	0x00000008
#define AOFF_mm_arg_start	0x000000a0
#define ASIZ_mm_arg_start	0x00000008
#define AOFF_mm_arg_end	0x000000a8
#define ASIZ_mm_arg_end	0x00000008
#define AOFF_mm_env_start	0x000000b0
#define ASIZ_mm_env_start	0x00000008
#define AOFF_mm_env_end	0x000000b8
#define ASIZ_mm_env_end	0x00000008
#define AOFF_mm_rss	0x000000c0
#define ASIZ_mm_rss	0x00000008
#define AOFF_mm_total_vm	0x000000c8
#define ASIZ_mm_total_vm	0x00000008
#define AOFF_mm_locked_vm	0x000000d0
#define ASIZ_mm_locked_vm	0x00000008
#define AOFF_mm_def_flags	0x000000d8
#define ASIZ_mm_def_flags	0x00000008
#define AOFF_mm_cpu_vm_mask	0x000000e0
#define ASIZ_mm_cpu_vm_mask	0x00000008
#define AOFF_mm_swap_cnt	0x000000e8
#define ASIZ_mm_swap_cnt	0x00000008
#define AOFF_mm_swap_address	0x000000f0
#define ASIZ_mm_swap_address	0x00000008
#define AOFF_mm_segments	0x000000f8
#define ASIZ_mm_segments	0x00000008
#define ASIZ_mm	0x00000100
#define AOFF_thread_ksp	0x00000000
#define ASIZ_thread_ksp	0x00000008
#define AOFF_thread_wstate	0x00000008
#define ASIZ_thread_wstate	0x00000001
#define AOFF_thread_cwp	0x00000009
#define ASIZ_thread_cwp	0x00000001
#define AOFF_thread_flags	0x0000000a
#define ASIZ_thread_flags	0x00000001
#define AOFF_thread_current_ds	0x0000000b
#define ASIZ_thread_current_ds	0x00000001
#define AOFF_thread_w_saved	0x0000000c
#define ASIZ_thread_w_saved	0x00000001
#define AOFF_thread_fpdepth	0x0000000d
#define ASIZ_thread_fpdepth	0x00000001
#define AOFF_thread_fpsaved	0x0000000e
#define ASIZ_thread_fpsaved	0x00000007
#define AOFF_thread___pad1	0x00000015
#define ASIZ_thread___pad1	0x00000003
#define AOFF_thread_kregs	0x00000018
#define ASIZ_thread_kregs	0x00000008
#define AOFF_thread_utraps	0x00000020
#define ASIZ_thread_utraps	0x00000008
#define AOFF_thread_gsr	0x00000028
#define ASIZ_thread_gsr	0x00000007
#define AOFF_thread___pad2	0x0000002f
#define ASIZ_thread___pad2	0x00000001
#define AOFF_thread_sig_address	0x00000030
#define ASIZ_thread_sig_address	0x00000008
#define AOFF_thread_sig_desc	0x00000038
#define ASIZ_thread_sig_desc	0x00000008
#define AOFF_thread_xfsr	0x00000040
#define ASIZ_thread_xfsr	0x00000038
#define AOFF_thread___pad3	0x00000078
#define ASIZ_thread___pad3	0x00000008
#define AOFF_thread_reg_window	0x00000080
#define ASIZ_thread_reg_window	0x00000380
#define AOFF_thread_rwbuf_stkptrs	0x00000400
#define ASIZ_thread_rwbuf_stkptrs	0x00000038
#define AOFF_thread_user_cntd0	0x00000438
#define ASIZ_thread_user_cntd0	0x00000008
#define AOFF_thread_user_cntd1	0x00000440
#define ASIZ_thread_user_cntd1	0x00000008
#define AOFF_thread_kernel_cntd0	0x00000448
#define ASIZ_thread_kernel_cntd0	0x00000008
#define AOFF_thread_kernel_cntd1	0x00000450
#define ASIZ_thread_kernel_cntd1	0x00000008
#define AOFF_thread_pcr_reg	0x00000458
#define ASIZ_thread_pcr_reg	0x00000008
#define ASIZ_thread	0x00000460

#else /* CONFIG_SMP */

#ifndef SPIN_LOCK_DEBUG

#define AOFF_task_state	0x00000000
#define ASIZ_task_state	0x00000008
#define AOFF_task_flags	0x00000008
#define ASIZ_task_flags	0x00000008
#define AOFF_task_sigpending	0x00000010
#define ASIZ_task_sigpending	0x00000004
#define AOFF_task_addr_limit	0x00000014
#define ASIZ_task_addr_limit	0x00000001
#define AOFF_task_exec_domain	0x00000018
#define ASIZ_task_exec_domain	0x00000008
#define AOFF_task_need_resched	0x00000020
#define ASIZ_task_need_resched	0x00000008
#define AOFF_task_counter	0x00000028
#define ASIZ_task_counter	0x00000008
#define AOFF_task_priority	0x00000030
#define ASIZ_task_priority	0x00000008
#define AOFF_task_avg_slice	0x00000038
#define ASIZ_task_avg_slice	0x00000008
#define AOFF_task_has_cpu	0x00000040
#define ASIZ_task_has_cpu	0x00000004
#define AOFF_task_processor	0x00000044
#define ASIZ_task_processor	0x00000004
#define AOFF_task_last_processor	0x00000048
#define ASIZ_task_last_processor	0x00000004
#define AOFF_task_lock_depth	0x0000004c
#define ASIZ_task_lock_depth	0x00000004
#define AOFF_task_next_task	0x00000050
#define ASIZ_task_next_task	0x00000008
#define AOFF_task_prev_task	0x00000058
#define ASIZ_task_prev_task	0x00000008
#define AOFF_task_run_list	0x00000060
#define ASIZ_task_run_list	0x00000010
#define AOFF_task_binfmt	0x00000070
#define ASIZ_task_binfmt	0x00000008
#define AOFF_task_exit_code	0x00000078
#define ASIZ_task_exit_code	0x00000004
#define AOFF_task_exit_signal	0x0000007c
#define ASIZ_task_exit_signal	0x00000004
#define AOFF_task_pdeath_signal	0x00000080
#define ASIZ_task_pdeath_signal	0x00000004
#define AOFF_task_personality	0x00000088
#define ASIZ_task_personality	0x00000008
#define AOFF_task_pid	0x00000094
#define ASIZ_task_pid	0x00000004
#define AOFF_task_pgrp	0x00000098
#define ASIZ_task_pgrp	0x00000004
#define AOFF_task_tty_old_pgrp	0x0000009c
#define ASIZ_task_tty_old_pgrp	0x00000004
#define AOFF_task_session	0x000000a0
#define ASIZ_task_session	0x00000004
#define AOFF_task_leader	0x000000a4
#define ASIZ_task_leader	0x00000004
#define AOFF_task_p_opptr	0x000000a8
#define ASIZ_task_p_opptr	0x00000008
#define AOFF_task_p_pptr	0x000000b0
#define ASIZ_task_p_pptr	0x00000008
#define AOFF_task_p_cptr	0x000000b8
#define ASIZ_task_p_cptr	0x00000008
#define AOFF_task_p_ysptr	0x000000c0
#define ASIZ_task_p_ysptr	0x00000008
#define AOFF_task_p_osptr	0x000000c8
#define ASIZ_task_p_osptr	0x00000008
#define AOFF_task_pidhash_next	0x000000d0
#define ASIZ_task_pidhash_next	0x00000008
#define AOFF_task_pidhash_pprev	0x000000d8
#define ASIZ_task_pidhash_pprev	0x00000008
#define AOFF_task_wait_chldexit	0x000000e0
#define ASIZ_task_wait_chldexit	0x00000028
#define AOFF_task_vfork_sem	0x00000108
#define ASIZ_task_vfork_sem	0x00000008
#define AOFF_task_policy	0x00000110
#define ASIZ_task_policy	0x00000008
#define AOFF_task_rt_priority	0x00000118
#define ASIZ_task_rt_priority	0x00000008
#define AOFF_task_it_real_value	0x00000120
#define ASIZ_task_it_real_value	0x00000008
#define AOFF_task_it_prof_value	0x00000128
#define ASIZ_task_it_prof_value	0x00000008
#define AOFF_task_it_virt_value	0x00000130
#define ASIZ_task_it_virt_value	0x00000008
#define AOFF_task_it_real_incr	0x00000138
#define ASIZ_task_it_real_incr	0x00000008
#define AOFF_task_it_prof_incr	0x00000140
#define ASIZ_task_it_prof_incr	0x00000008
#define AOFF_task_it_virt_incr	0x00000148
#define ASIZ_task_it_virt_incr	0x00000008
#define AOFF_task_real_timer	0x00000150
#define ASIZ_task_real_timer	0x00000028
#define AOFF_task_times	0x00000178
#define ASIZ_task_times	0x00000020
#define AOFF_task_start_time	0x00000198
#define ASIZ_task_start_time	0x00000008
#define AOFF_task_per_cpu_utime	0x000001a0
#define ASIZ_task_per_cpu_utime	0x00000100
#define AOFF_task_min_flt	0x000003a0
#define ASIZ_task_min_flt	0x00000008
#define AOFF_task_maj_flt	0x000003a8
#define ASIZ_task_maj_flt	0x00000008
#define AOFF_task_nswap	0x000003b0
#define ASIZ_task_nswap	0x00000008
#define AOFF_task_cmin_flt	0x000003b8
#define ASIZ_task_cmin_flt	0x00000008
#define AOFF_task_cmaj_flt	0x000003c0
#define ASIZ_task_cmaj_flt	0x00000008
#define AOFF_task_cnswap	0x000003c8
#define ASIZ_task_cnswap	0x00000008
#define AOFF_task_uid	0x000003d4
#define ASIZ_task_uid	0x00000004
#define AOFF_task_euid	0x000003d8
#define ASIZ_task_euid	0x00000004
#define AOFF_task_suid	0x000003dc
#define ASIZ_task_suid	0x00000004
#define AOFF_task_fsuid	0x000003e0
#define ASIZ_task_fsuid	0x00000004
#define AOFF_task_gid	0x000003e4
#define ASIZ_task_gid	0x00000004
#define AOFF_task_egid	0x000003e8
#define ASIZ_task_egid	0x00000004
#define AOFF_task_sgid	0x000003ec
#define ASIZ_task_sgid	0x00000004
#define AOFF_task_fsgid	0x000003f0
#define ASIZ_task_fsgid	0x00000004
#define AOFF_task_ngroups	0x000003f4
#define ASIZ_task_ngroups	0x00000004
#define AOFF_task_groups	0x000003f8
#define ASIZ_task_groups	0x00000080
#define AOFF_task_cap_effective	0x00000478
#define ASIZ_task_cap_effective	0x00000004
#define AOFF_task_cap_inheritable	0x0000047c
#define ASIZ_task_cap_inheritable	0x00000004
#define AOFF_task_cap_permitted	0x00000480
#define ASIZ_task_cap_permitted	0x00000004
#define AOFF_task_user	0x00000488
#define ASIZ_task_user	0x00000008
#define AOFF_task_rlim	0x00000490
#define ASIZ_task_rlim	0x000000a0
#define AOFF_task_used_math	0x00000530
#define ASIZ_task_used_math	0x00000002
#define AOFF_task_comm	0x00000532
#define ASIZ_task_comm	0x00000010
#define AOFF_task_link_count	0x00000544
#define ASIZ_task_link_count	0x00000004
#define AOFF_task_tty	0x00000548
#define ASIZ_task_tty	0x00000008
#define AOFF_task_semundo	0x00000550
#define ASIZ_task_semundo	0x00000008
#define AOFF_task_semsleeping	0x00000558
#define ASIZ_task_semsleeping	0x00000008
#define AOFF_task_thread	0x00000560
#define ASIZ_task_thread	0x00000460
#define AOFF_task_fs	0x000009c0
#define ASIZ_task_fs	0x00000008
#define AOFF_task_files	0x000009c8
#define ASIZ_task_files	0x00000008
#define AOFF_task_mm	0x000009d0
#define ASIZ_task_mm	0x00000008
#define AOFF_task_active_mm	0x000009d8
#define ASIZ_task_active_mm	0x00000008
#define AOFF_task_sigmask_lock	0x000009e0
#define ASIZ_task_sigmask_lock	0x00000001
#define AOFF_task_sig	0x000009e8
#define ASIZ_task_sig	0x00000008
#define AOFF_task_signal	0x000009f0
#define ASIZ_task_signal	0x00000008
#define AOFF_task_blocked	0x000009f8
#define ASIZ_task_blocked	0x00000008
#define AOFF_task_sigqueue	0x00000a00
#define ASIZ_task_sigqueue	0x00000008
#define AOFF_task_sigqueue_tail	0x00000a08
#define ASIZ_task_sigqueue_tail	0x00000008
#define AOFF_task_sas_ss_sp	0x00000a10
#define ASIZ_task_sas_ss_sp	0x00000008
#define AOFF_task_sas_ss_size	0x00000a18
#define ASIZ_task_sas_ss_size	0x00000008
#define ASIZ_task	0x00000a20
#define AOFF_mm_mmap	0x00000000
#define ASIZ_mm_mmap	0x00000008
#define AOFF_mm_mmap_avl	0x00000008
#define ASIZ_mm_mmap_avl	0x00000008
#define AOFF_mm_mmap_cache	0x00000010
#define ASIZ_mm_mmap_cache	0x00000008
#define AOFF_mm_pgd	0x00000018
#define ASIZ_mm_pgd	0x00000008
#define AOFF_mm_mm_users	0x00000020
#define ASIZ_mm_mm_users	0x00000004
#define AOFF_mm_mm_count	0x00000024
#define ASIZ_mm_mm_count	0x00000004
#define AOFF_mm_map_count	0x00000028
#define ASIZ_mm_map_count	0x00000004
#define AOFF_mm_mmap_sem	0x00000030
#define ASIZ_mm_mmap_sem	0x00000038
#define AOFF_mm_page_table_lock	0x00000068
#define ASIZ_mm_page_table_lock	0x00000001
#define AOFF_mm_context	0x00000070
#define ASIZ_mm_context	0x00000008
#define AOFF_mm_start_code	0x00000078
#define ASIZ_mm_start_code	0x00000008
#define AOFF_mm_end_code	0x00000080
#define ASIZ_mm_end_code	0x00000008
#define AOFF_mm_start_data	0x00000088
#define ASIZ_mm_start_data	0x00000008
#define AOFF_mm_end_data	0x00000090
#define ASIZ_mm_end_data	0x00000008
#define AOFF_mm_start_brk	0x00000098
#define ASIZ_mm_start_brk	0x00000008
#define AOFF_mm_brk	0x000000a0
#define ASIZ_mm_brk	0x00000008
#define AOFF_mm_start_stack	0x000000a8
#define ASIZ_mm_start_stack	0x00000008
#define AOFF_mm_arg_start	0x000000b0
#define ASIZ_mm_arg_start	0x00000008
#define AOFF_mm_arg_end	0x000000b8
#define ASIZ_mm_arg_end	0x00000008
#define AOFF_mm_env_start	0x000000c0
#define ASIZ_mm_env_start	0x00000008
#define AOFF_mm_env_end	0x000000c8
#define ASIZ_mm_env_end	0x00000008
#define AOFF_mm_rss	0x000000d0
#define ASIZ_mm_rss	0x00000008
#define AOFF_mm_total_vm	0x000000d8
#define ASIZ_mm_total_vm	0x00000008
#define AOFF_mm_locked_vm	0x000000e0
#define ASIZ_mm_locked_vm	0x00000008
#define AOFF_mm_def_flags	0x000000e8
#define ASIZ_mm_def_flags	0x00000008
#define AOFF_mm_cpu_vm_mask	0x000000f0
#define ASIZ_mm_cpu_vm_mask	0x00000008
#define AOFF_mm_swap_cnt	0x000000f8
#define ASIZ_mm_swap_cnt	0x00000008
#define AOFF_mm_swap_address	0x00000100
#define ASIZ_mm_swap_address	0x00000008
#define AOFF_mm_segments	0x00000108
#define ASIZ_mm_segments	0x00000008
#define ASIZ_mm	0x00000110
#define AOFF_thread_ksp	0x00000000
#define ASIZ_thread_ksp	0x00000008
#define AOFF_thread_wstate	0x00000008
#define ASIZ_thread_wstate	0x00000001
#define AOFF_thread_cwp	0x00000009
#define ASIZ_thread_cwp	0x00000001
#define AOFF_thread_flags	0x0000000a
#define ASIZ_thread_flags	0x00000001
#define AOFF_thread_current_ds	0x0000000b
#define ASIZ_thread_current_ds	0x00000001
#define AOFF_thread_w_saved	0x0000000c
#define ASIZ_thread_w_saved	0x00000001
#define AOFF_thread_fpdepth	0x0000000d
#define ASIZ_thread_fpdepth	0x00000001
#define AOFF_thread_fpsaved	0x0000000e
#define ASIZ_thread_fpsaved	0x00000007
#define AOFF_thread___pad1	0x00000015
#define ASIZ_thread___pad1	0x00000003
#define AOFF_thread_kregs	0x00000018
#define ASIZ_thread_kregs	0x00000008
#define AOFF_thread_utraps	0x00000020
#define ASIZ_thread_utraps	0x00000008
#define AOFF_thread_gsr	0x00000028
#define ASIZ_thread_gsr	0x00000007
#define AOFF_thread___pad2	0x0000002f
#define ASIZ_thread___pad2	0x00000001
#define AOFF_thread_sig_address	0x00000030
#define ASIZ_thread_sig_address	0x00000008
#define AOFF_thread_sig_desc	0x00000038
#define ASIZ_thread_sig_desc	0x00000008
#define AOFF_thread_xfsr	0x00000040
#define ASIZ_thread_xfsr	0x00000038
#define AOFF_thread___pad3	0x00000078
#define ASIZ_thread___pad3	0x00000008
#define AOFF_thread_reg_window	0x00000080
#define ASIZ_thread_reg_window	0x00000380
#define AOFF_thread_rwbuf_stkptrs	0x00000400
#define ASIZ_thread_rwbuf_stkptrs	0x00000038
#define AOFF_thread_user_cntd0	0x00000438
#define ASIZ_thread_user_cntd0	0x00000008
#define AOFF_thread_user_cntd1	0x00000440
#define ASIZ_thread_user_cntd1	0x00000008
#define AOFF_thread_kernel_cntd0	0x00000448
#define ASIZ_thread_kernel_cntd0	0x00000008
#define AOFF_thread_kernel_cntd1	0x00000450
#define ASIZ_thread_kernel_cntd1	0x00000008
#define AOFF_thread_pcr_reg	0x00000458
#define ASIZ_thread_pcr_reg	0x00000008
#define ASIZ_thread	0x00000460

#else /* SPIN_LOCK_DEBUG */

#define AOFF_task_state	0x00000000
#define ASIZ_task_state	0x00000008
#define AOFF_task_flags	0x00000008
#define ASIZ_task_flags	0x00000008
#define AOFF_task_sigpending	0x00000010
#define ASIZ_task_sigpending	0x00000004
#define AOFF_task_addr_limit	0x00000014
#define ASIZ_task_addr_limit	0x00000001
#define AOFF_task_exec_domain	0x00000018
#define ASIZ_task_exec_domain	0x00000008
#define AOFF_task_need_resched	0x00000020
#define ASIZ_task_need_resched	0x00000008
#define AOFF_task_counter	0x00000028
#define ASIZ_task_counter	0x00000008
#define AOFF_task_priority	0x00000030
#define ASIZ_task_priority	0x00000008
#define AOFF_task_avg_slice	0x00000038
#define ASIZ_task_avg_slice	0x00000008
#define AOFF_task_has_cpu	0x00000040
#define ASIZ_task_has_cpu	0x00000004
#define AOFF_task_processor	0x00000044
#define ASIZ_task_processor	0x00000004
#define AOFF_task_last_processor	0x00000048
#define ASIZ_task_last_processor	0x00000004
#define AOFF_task_lock_depth	0x0000004c
#define ASIZ_task_lock_depth	0x00000004
#define AOFF_task_next_task	0x00000050
#define ASIZ_task_next_task	0x00000008
#define AOFF_task_prev_task	0x00000058
#define ASIZ_task_prev_task	0x00000008
#define AOFF_task_run_list	0x00000060
#define ASIZ_task_run_list	0x00000010
#define AOFF_task_binfmt	0x00000070
#define ASIZ_task_binfmt	0x00000008
#define AOFF_task_exit_code	0x00000078
#define ASIZ_task_exit_code	0x00000004
#define AOFF_task_exit_signal	0x0000007c
#define ASIZ_task_exit_signal	0x00000004
#define AOFF_task_pdeath_signal	0x00000080
#define ASIZ_task_pdeath_signal	0x00000004
#define AOFF_task_personality	0x00000088
#define ASIZ_task_personality	0x00000008
#define AOFF_task_pid	0x00000094
#define ASIZ_task_pid	0x00000004
#define AOFF_task_pgrp	0x00000098
#define ASIZ_task_pgrp	0x00000004
#define AOFF_task_tty_old_pgrp	0x0000009c
#define ASIZ_task_tty_old_pgrp	0x00000004
#define AOFF_task_session	0x000000a0
#define ASIZ_task_session	0x00000004
#define AOFF_task_leader	0x000000a4
#define ASIZ_task_leader	0x00000004
#define AOFF_task_p_opptr	0x000000a8
#define ASIZ_task_p_opptr	0x00000008
#define AOFF_task_p_pptr	0x000000b0
#define ASIZ_task_p_pptr	0x00000008
#define AOFF_task_p_cptr	0x000000b8
#define ASIZ_task_p_cptr	0x00000008
#define AOFF_task_p_ysptr	0x000000c0
#define ASIZ_task_p_ysptr	0x00000008
#define AOFF_task_p_osptr	0x000000c8
#define ASIZ_task_p_osptr	0x00000008
#define AOFF_task_pidhash_next	0x000000d0
#define ASIZ_task_pidhash_next	0x00000008
#define AOFF_task_pidhash_pprev	0x000000d8
#define ASIZ_task_pidhash_pprev	0x00000008
#define AOFF_task_wait_chldexit	0x000000e0
#define ASIZ_task_wait_chldexit	0x00000030
#define AOFF_task_vfork_sem	0x00000110
#define ASIZ_task_vfork_sem	0x00000008
#define AOFF_task_policy	0x00000118
#define ASIZ_task_policy	0x00000008
#define AOFF_task_rt_priority	0x00000120
#define ASIZ_task_rt_priority	0x00000008
#define AOFF_task_it_real_value	0x00000128
#define ASIZ_task_it_real_value	0x00000008
#define AOFF_task_it_prof_value	0x00000130
#define ASIZ_task_it_prof_value	0x00000008
#define AOFF_task_it_virt_value	0x00000138
#define ASIZ_task_it_virt_value	0x00000008
#define AOFF_task_it_real_incr	0x00000140
#define ASIZ_task_it_real_incr	0x00000008
#define AOFF_task_it_prof_incr	0x00000148
#define ASIZ_task_it_prof_incr	0x00000008
#define AOFF_task_it_virt_incr	0x00000150
#define ASIZ_task_it_virt_incr	0x00000008
#define AOFF_task_real_timer	0x00000158
#define ASIZ_task_real_timer	0x00000028
#define AOFF_task_times	0x00000180
#define ASIZ_task_times	0x00000020
#define AOFF_task_start_time	0x000001a0
#define ASIZ_task_start_time	0x00000008
#define AOFF_task_per_cpu_utime	0x000001a8
#define ASIZ_task_per_cpu_utime	0x00000100
#define AOFF_task_min_flt	0x000003a8
#define ASIZ_task_min_flt	0x00000008
#define AOFF_task_maj_flt	0x000003b0
#define ASIZ_task_maj_flt	0x00000008
#define AOFF_task_nswap	0x000003b8
#define ASIZ_task_nswap	0x00000008
#define AOFF_task_cmin_flt	0x000003c0
#define ASIZ_task_cmin_flt	0x00000008
#define AOFF_task_cmaj_flt	0x000003c8
#define ASIZ_task_cmaj_flt	0x00000008
#define AOFF_task_cnswap	0x000003d0
#define ASIZ_task_cnswap	0x00000008
#define AOFF_task_uid	0x000003dc
#define ASIZ_task_uid	0x00000004
#define AOFF_task_euid	0x000003e0
#define ASIZ_task_euid	0x00000004
#define AOFF_task_suid	0x000003e4
#define ASIZ_task_suid	0x00000004
#define AOFF_task_fsuid	0x000003e8
#define ASIZ_task_fsuid	0x00000004
#define AOFF_task_gid	0x000003ec
#define ASIZ_task_gid	0x00000004
#define AOFF_task_egid	0x000003f0
#define ASIZ_task_egid	0x00000004
#define AOFF_task_sgid	0x000003f4
#define ASIZ_task_sgid	0x00000004
#define AOFF_task_fsgid	0x000003f8
#define ASIZ_task_fsgid	0x00000004
#define AOFF_task_ngroups	0x000003fc
#define ASIZ_task_ngroups	0x00000004
#define AOFF_task_groups	0x00000400
#define ASIZ_task_groups	0x00000080
#define AOFF_task_cap_effective	0x00000480
#define ASIZ_task_cap_effective	0x00000004
#define AOFF_task_cap_inheritable	0x00000484
#define ASIZ_task_cap_inheritable	0x00000004
#define AOFF_task_cap_permitted	0x00000488
#define ASIZ_task_cap_permitted	0x00000004
#define AOFF_task_user	0x00000490
#define ASIZ_task_user	0x00000008
#define AOFF_task_rlim	0x00000498
#define ASIZ_task_rlim	0x000000a0
#define AOFF_task_used_math	0x00000538
#define ASIZ_task_used_math	0x00000002
#define AOFF_task_comm	0x0000053a
#define ASIZ_task_comm	0x00000010
#define AOFF_task_link_count	0x0000054c
#define ASIZ_task_link_count	0x00000004
#define AOFF_task_tty	0x00000550
#define ASIZ_task_tty	0x00000008
#define AOFF_task_semundo	0x00000558
#define ASIZ_task_semundo	0x00000008
#define AOFF_task_semsleeping	0x00000560
#define ASIZ_task_semsleeping	0x00000008
#define AOFF_task_thread	0x00000570
#define ASIZ_task_thread	0x00000460
#define AOFF_task_fs	0x000009d0
#define ASIZ_task_fs	0x00000008
#define AOFF_task_files	0x000009d8
#define ASIZ_task_files	0x00000008
#define AOFF_task_mm	0x000009e0
#define ASIZ_task_mm	0x00000008
#define AOFF_task_active_mm	0x000009e8
#define ASIZ_task_active_mm	0x00000008
#define AOFF_task_sigmask_lock	0x000009f0
#define ASIZ_task_sigmask_lock	0x0000000c
#define AOFF_task_sig	0x00000a00
#define ASIZ_task_sig	0x00000008
#define AOFF_task_signal	0x00000a08
#define ASIZ_task_signal	0x00000008
#define AOFF_task_blocked	0x00000a10
#define ASIZ_task_blocked	0x00000008
#define AOFF_task_sigqueue	0x00000a18
#define ASIZ_task_sigqueue	0x00000008
#define AOFF_task_sigqueue_tail	0x00000a20
#define ASIZ_task_sigqueue_tail	0x00000008
#define AOFF_task_sas_ss_sp	0x00000a28
#define ASIZ_task_sas_ss_sp	0x00000008
#define AOFF_task_sas_ss_size	0x00000a30
#define ASIZ_task_sas_ss_size	0x00000008
#define ASIZ_task	0x00000a40
#define AOFF_mm_mmap	0x00000000
#define ASIZ_mm_mmap	0x00000008
#define AOFF_mm_mmap_avl	0x00000008
#define ASIZ_mm_mmap_avl	0x00000008
#define AOFF_mm_mmap_cache	0x00000010
#define ASIZ_mm_mmap_cache	0x00000008
#define AOFF_mm_pgd	0x00000018
#define ASIZ_mm_pgd	0x00000008
#define AOFF_mm_mm_users	0x00000020
#define ASIZ_mm_mm_users	0x00000004
#define AOFF_mm_mm_count	0x00000024
#define ASIZ_mm_mm_count	0x00000004
#define AOFF_mm_map_count	0x00000028
#define ASIZ_mm_map_count	0x00000004
#define AOFF_mm_mmap_sem	0x00000030
#define ASIZ_mm_mmap_sem	0x00000040
#define AOFF_mm_page_table_lock	0x00000070
#define ASIZ_mm_page_table_lock	0x0000000c
#define AOFF_mm_context	0x00000080
#define ASIZ_mm_context	0x00000008
#define AOFF_mm_start_code	0x00000088
#define ASIZ_mm_start_code	0x00000008
#define AOFF_mm_end_code	0x00000090
#define ASIZ_mm_end_code	0x00000008
#define AOFF_mm_start_data	0x00000098
#define ASIZ_mm_start_data	0x00000008
#define AOFF_mm_end_data	0x000000a0
#define ASIZ_mm_end_data	0x00000008
#define AOFF_mm_start_brk	0x000000a8
#define ASIZ_mm_start_brk	0x00000008
#define AOFF_mm_brk	0x000000b0
#define ASIZ_mm_brk	0x00000008
#define AOFF_mm_start_stack	0x000000b8
#define ASIZ_mm_start_stack	0x00000008
#define AOFF_mm_arg_start	0x000000c0
#define ASIZ_mm_arg_start	0x00000008
#define AOFF_mm_arg_end	0x000000c8
#define ASIZ_mm_arg_end	0x00000008
#define AOFF_mm_env_start	0x000000d0
#define ASIZ_mm_env_start	0x00000008
#define AOFF_mm_env_end	0x000000d8
#define ASIZ_mm_env_end	0x00000008
#define AOFF_mm_rss	0x000000e0
#define ASIZ_mm_rss	0x00000008
#define AOFF_mm_total_vm	0x000000e8
#define ASIZ_mm_total_vm	0x00000008
#define AOFF_mm_locked_vm	0x000000f0
#define ASIZ_mm_locked_vm	0x00000008
#define AOFF_mm_def_flags	0x000000f8
#define ASIZ_mm_def_flags	0x00000008
#define AOFF_mm_cpu_vm_mask	0x00000100
#define ASIZ_mm_cpu_vm_mask	0x00000008
#define AOFF_mm_swap_cnt	0x00000108
#define ASIZ_mm_swap_cnt	0x00000008
#define AOFF_mm_swap_address	0x00000110
#define ASIZ_mm_swap_address	0x00000008
#define AOFF_mm_segments	0x00000118
#define ASIZ_mm_segments	0x00000008
#define ASIZ_mm	0x00000120
#define AOFF_thread_ksp	0x00000000
#define ASIZ_thread_ksp	0x00000008
#define AOFF_thread_wstate	0x00000008
#define ASIZ_thread_wstate	0x00000001
#define AOFF_thread_cwp	0x00000009
#define ASIZ_thread_cwp	0x00000001
#define AOFF_thread_flags	0x0000000a
#define ASIZ_thread_flags	0x00000001
#define AOFF_thread_current_ds	0x0000000b
#define ASIZ_thread_current_ds	0x00000001
#define AOFF_thread_w_saved	0x0000000c
#define ASIZ_thread_w_saved	0x00000001
#define AOFF_thread_fpdepth	0x0000000d
#define ASIZ_thread_fpdepth	0x00000001
#define AOFF_thread_fpsaved	0x0000000e
#define ASIZ_thread_fpsaved	0x00000007
#define AOFF_thread___pad1	0x00000015
#define ASIZ_thread___pad1	0x00000003
#define AOFF_thread_kregs	0x00000018
#define ASIZ_thread_kregs	0x00000008
#define AOFF_thread_utraps	0x00000020
#define ASIZ_thread_utraps	0x00000008
#define AOFF_thread_gsr	0x00000028
#define ASIZ_thread_gsr	0x00000007
#define AOFF_thread___pad2	0x0000002f
#define ASIZ_thread___pad2	0x00000001
#define AOFF_thread_sig_address	0x00000030
#define ASIZ_thread_sig_address	0x00000008
#define AOFF_thread_sig_desc	0x00000038
#define ASIZ_thread_sig_desc	0x00000008
#define AOFF_thread_xfsr	0x00000040
#define ASIZ_thread_xfsr	0x00000038
#define AOFF_thread___pad3	0x00000078
#define ASIZ_thread___pad3	0x00000008
#define AOFF_thread_reg_window	0x00000080
#define ASIZ_thread_reg_window	0x00000380
#define AOFF_thread_rwbuf_stkptrs	0x00000400
#define ASIZ_thread_rwbuf_stkptrs	0x00000038
#define AOFF_thread_user_cntd0	0x00000438
#define ASIZ_thread_user_cntd0	0x00000008
#define AOFF_thread_user_cntd1	0x00000440
#define ASIZ_thread_user_cntd1	0x00000008
#define AOFF_thread_kernel_cntd0	0x00000448
#define ASIZ_thread_kernel_cntd0	0x00000008
#define AOFF_thread_kernel_cntd1	0x00000450
#define ASIZ_thread_kernel_cntd1	0x00000008
#define AOFF_thread_pcr_reg	0x00000458
#define ASIZ_thread_pcr_reg	0x00000008
#define ASIZ_thread	0x00000460
#endif /* SPIN_LOCK_DEBUG */

#endif /* CONFIG_SMP */

#endif /* __ASM_OFFSETS_H__ */
